#include "wrapper.h"
#include "nanotec.h"
#include "TT.h"


//--------------------------------------------------------------------------------------------------
map U16 Controlword as inout 0x6040:00
map U16 Statusword as input 0x6041:00
map U32 Inputs as input 0x60FD:00
map U32 Outputs as inout 0x60FE:01
map S08 ModesOfOperation as output 0x6060:00
map S08 ModesOfOperationDisplay as input 0x6061:00
map S16 AnalogInput as input 0x3220:01
map S32 ActualPosition as input 0x6064:00
map S32 TargetPosition as inout 0x607A:00
map S32 TargetVelocity as inout 0x60FF:00
map U32 ProfileVelocity as inout 0x6081:00
map U32 ProfileAcceleration as inout 0x6083:00
map U32 ProfileDeceleration as inout 0x6084:00
map U32 Storage1 as inout 0x2701:01
map U32 Storage2 as inout 0x2701:02
map S32 DemandPosition as input 0x6062:00

// Entr�es TOR :
//--------------------------------------------------------------------------------------------------
static bool IN1; static bool IN1_prec; static bool IN1_up; static bool IN1_dw;
static bool IN2; static bool IN2_prec; static bool IN2_up; static bool IN2_dw;
static bool IN3; static bool IN3_prec; static bool IN3_up; static bool IN3_dw;
static bool IN4; static bool IN4_prec; static bool IN4_up; static bool IN4_dw;
static bool IN5; static bool IN5_prec; static bool IN5_up; static bool IN5_dw;
static bool IN6; static bool IN6_prec; static bool IN6_up; static bool IN6_dw;

// Sorties TOR :
//--------------------------------------------------------------------------------------------------
static bool OP1;
static bool OP2;
//--------------------------------------------------------------------------------------------------

// Graphcet de mouvement :
//--------------------------------------------------------------------------------------------------
static S16 move_step_set;
static S16 move_step_act;
static bool move_step_new;


// Etat du moteur :
//--------------------------------------------------------------------------------------------------
#define dsNotReadyToSwitchOn	1
#define dsSwitchOnDisabled		2
#define dsReadyToSwitchOn		3
#define dsSwitchedOn			4
#define dsOperationEnabled		5
#define dsQuickStopActive		6
#define dsFaultReactionActive	7
#define dsFault					8
static U08 dsSTATE;

// Commandes du moteur :
//--------------------------------------------------------------------------------------------------
#define dsShutdown				1
#define dsSwitchOn				2
#define dsDisableVoltage		3
#define dsQuickStop				4
#define dsDisableOperation		5
#define dsEnableOperation		6
#define dsFaultReset			7
static U08 dsCOMMAND;

// D�claration des fonctions :
//--------------------------------------------------------------------------------------------------
void init();
void ctrl();
void ctrl_rd();
void ctrl_plc();
void ctrl_plc_enable_reset();
void ctrl_plc_move();
void ctrl_plc_op();
void ctrl_wr();

// G�n�ral :
//--------------------------------------------------------------------------------------------------
static bool allHalt;
//--------------------------------------------------------------------------------------------------

// Profile Position : 
//--------------------------------------------------------------------------------------------------
static bool ppTargetReached;
static bool ppSetPointAck;
//--------------------------------------------------------------------------------------------------
static bool ppNewSetPoint;
static bool ppChangeSetImmediately;
static bool ppRelative;

// Profile Velocity :
//--------------------------------------------------------------------------------------------------
static bool pvTargetSpeedReached;
//--------------------------------------------------------------------------------------------------


// Programme principal :
//--------------------------------------------------------------------------------------------------
void user()
{

	init();
			
	while( true )
	{
		ctrl();		
		yield();
	}
}

// Initialisation :
//--------------------------------------------------------------------------------------------------
void init()
{

	// S�lection du mode de marche = Profile Velocity :
	//----------------------------------------------------------------------------------------------
	Out.ModesOfOperation = 3;
	
	// Initialisation des �tapes graphcet :
	//----------------------------------------------------------------------------------------------
	move_step_act = -1;
	move_step_set = 0;
	move_step_new = false;
	
	// Param�tres de mouvement par d�faut :
	//----------------------------------------------------------------------------------------------
	InOut.TargetVelocity = 0;
	InOut.ProfileAcceleration = 50;
	InOut.ProfileDeceleration = 50;
	
	// Configuration des param�tres :
	//----------------------------------------------------------------------------------------------
od_write(0x6075,0,1400);

}
// T�che plc :
//--------------------------------------------------------------------------------------------------
void ctrl()
{
	ctrl_rd();
	ctrl_plc();
	ctrl_wr();
}


// Lecture des entr�es et de l'�tat du moteur :
//--------------------------------------------------------------------------------------------------
void ctrl_rd()
{
	// Lecture de l'�tat des entr�es TOR :
	//----------------------------------------------------------------------------------------------
	IN1 = DigitalInput(1);
	IN2 = DigitalInput(2);
	IN3 = DigitalInput(3);
	IN4 = DigitalInput(4);
	IN5 = DigitalInput(5);
	IN6 = DigitalInput(6);
	
	// Fronts montants / descendants :
	//----------------------------------------------------------------------------------------------
	IN1_up = bit_up( IN1 , IN1_prec); IN1_dw = bit_dw( IN1 , IN1_prec ); IN1_prec = IN1;
	IN2_up = bit_up( IN2 , IN2_prec); IN2_dw = bit_dw( IN2 , IN2_prec ); IN2_prec = IN2;
	IN3_up = bit_up( IN3 , IN3_prec); IN3_dw = bit_dw( IN3 , IN3_prec ); IN3_prec = IN3;
	IN4_up = bit_up( IN4 , IN4_prec); IN4_dw = bit_dw( IN4 , IN4_prec ); IN4_prec = IN4;
	IN5_up = bit_up( IN5 , IN5_prec); IN5_dw = bit_dw( IN5 , IN5_prec ); IN5_prec = IN5;
	IN6_up = bit_up( IN6 , IN6_prec); IN6_dw = bit_dw( IN6 , IN6_prec ); IN6_prec = IN6;
	
	// Etat du moteur :
	//----------------------------------------------------------------------------------------------
	if ( ( In.Statusword & 0x004F ) == 0x0000 ) dsSTATE = dsNotReadyToSwitchOn;		
	else if ( ( In.Statusword & 0x004F ) == 0x0040 ) dsSTATE = dsSwitchOnDisabled;
	else if ( ( In.Statusword & 0x006F ) == 0x0021 ) dsSTATE = dsReadyToSwitchOn;
	else if ( ( In.Statusword & 0x006F ) == 0x0023 ) dsSTATE = dsSwitchedOn;
	else if ( ( In.Statusword & 0x006F ) == 0x0027 ) dsSTATE = dsOperationEnabled;
	else if ( ( In.Statusword & 0x006F ) == 0x0007 ) dsSTATE = dsQuickStopActive;
	else if ( ( In.Statusword & 0x004F ) == 0x000F ) dsSTATE = dsFaultReactionActive;
	else if ( ( In.Statusword & 0x004F ) == 0x0080 ) dsSTATE = dsFault;
	
	// Etat du mode de marche :
	//----------------------------------------------------------------------------------------------
	ppTargetReached = ( In.ModesOfOperationDisplay == 1 ) && ( In.Statusword & 0x0400 ); // bit 10 = Target Reached
	ppSetPointAck = ( In.ModesOfOperationDisplay == 1 ) && ( In.Statusword & 0x1000 ); // bit 12 = Set-point acknowledge
	pvTargetSpeedReached = ( In.ModesOfOperationDisplay == 3 ) && ( In.Statusword & 0x0400 );	// bit 10 = Target Speed reached
		
}

// Process :
//--------------------------------------------------------------------------------------------------
void ctrl_plc()
{

	// Enable + Reset Fault :
	//----------------------------------------------------------------------------------------------
	ctrl_plc_enable_reset();

	// Graphcet :
	//----------------------------------------------------------------------------------------------
	ctrl_plc_move();

	// Signaux de sorties :
	//-----------------------------------------------------------------------------------------------
	ctrl_plc_op();


}

// Mise en r�gulation / Reset d�faut :
//--------------------------------------------------------------------------------------------------
void ctrl_plc_enable_reset()
{

	dsCOMMAND = 0;
	
	bool Enable = true;
	bool Reset = false;
	
	
	switch ( dsSTATE )
	{
		case dsNotReadyToSwitchOn :
			break;
			
		case dsSwitchOnDisabled	:
			if ( Enable ) dsCOMMAND = dsShutdown;				// transititon 2 : Switch On Disabled --> Ready To Switch On
			break;
		
		case dsReadyToSwitchOn	:
			if ( Enable ) dsCOMMAND = dsSwitchOn;				// transition 3 : Ready To Switch On --> Switched On
			else dsCOMMAND = dsDisableVoltage;				// transition 7 : Ready To Switch On --> Switch On Disabled
			break;
			
		case dsSwitchedOn	:
			if ( Enable ) dsCOMMAND = dsEnableOperation;		// transition 4 : Switched On --> Operation Enabled
			else dsCOMMAND = dsShutdown;					// transition 6 : Switched On --> Ready To Switch On
			break;
			
		case dsOperationEnabled	:
			if ( !Enable ) dsCOMMAND =  dsDisableOperation;	// transition 5 : Operation Enabled --> Switched On
			break;
			
		case dsQuickStopActive	:
			break;
			
		case dsFaultReactionActive	:
			break;
			
		case dsFault	:
			if ( Reset ) dsCOMMAND = dsFaultReset;			// transition 15 : Fault --> Switch On Disabled
			break;
		
	}

}

// Process des commandes de mouvement :
//--------------------------------------------------------------------------------------------------
void ctrl_plc_move()
{
	
	// Contr�le changement d'�tape :
	//----------------------------------------------------------------------------------------------
	move_step_new = ( move_step_set != move_step_act );
	move_step_act = move_step_set;
	
	// For�age �tape initiale :
	//----------------------------------------------------------------------------------------------
	if ( dsSTATE != dsOperationEnabled ) move_step_set = 0;
	
	switch ( move_step_act )
	{
		// [[ 000 ]]----( wait:operation enabled )
		//		+ dsSTATE = dsOperationEnabled												--> �tape 100
		//-------------------------------------------------------------------------------------------
		case 0	:
			if ( dsSTATE == dsOperationEnabled ) move_step_set = 100;

			break;
			
		// [ 100 ]----( wait:dcy )
		//		+ IN1 																		--> �tape 200
		//		+ IN2																		--> �tape 300
		//-------------------------------------------------------------------------------------------
		case 100 :
			if ( IN1 ) move_step_set = 200;
			else if ( IN2 ) move_step_set = 300;
			break;										
			
		// [ 200 ]----( manu:rot+ )
		//		+ /IN1																		--> �tape 100 
		//-------------------------------------------------------------------------------------------
		case 200	:
			if ( move_step_new )
			{ 
				InOut.TargetVelocity = 20;
			} 
			if ( !IN1 ) 
			{
				move_step_set = 100;
			}
			break;
		
		// [ 300 ]----( manu:rot- )
		//		+ /IN2																		--> �tape 100
		//-------------------------------------------------------------------------------------------
		case 300	:
			if ( move_step_new )
			{ 
				InOut.TargetVelocity = -20;
			} 
			if ( !IN2 ) 
			{
				move_step_set = 100;
			}	
			break;				
	} 
	
	allHalt = ( move_step_act != 200 ) && ( move_step_act != 300 );  
		
}

// Process des signaux de sortie :
//--------------------------------------------------------------------------------------------------
void ctrl_plc_op()
{
	OP1 = ( dsSTATE == dsOperationEnabled );
	OP2 = ( dsSTATE == dsOperationEnabled ) && ( In.DemandPosition == 0 );	
}


// Ecriture des sorties et des commandes :
//--------------------------------------------------------------------------------------------------
void ctrl_wr()
{

	// Application de la commande dans le control word :
	//----------------------------------------------------------------------------------------------
	switch ( dsCOMMAND )
	{
		case dsShutdown	:
			InOut.Controlword = ( InOut.Controlword & 0xFF78 ) | 0x0006;   		
			break;
		case dsSwitchOn	:
			InOut.Controlword = ( InOut.Controlword & 0xFF70 ) | 0x0007;
			break;
		case dsDisableVoltage	:
			InOut.Controlword = ( InOut.Controlword & 0xFF7D ) | 0x0000;
			break;
		case dsQuickStop	:
			InOut.Controlword = ( InOut.Controlword & 0xFF79 ) | 0x0002;
			break;
		case dsDisableOperation	:
			InOut.Controlword = ( InOut.Controlword & 0xFF70 ) | 0x0007;
			break;
		case dsEnableOperation	:
			InOut.Controlword = ( InOut.Controlword & 0xFF70 ) | 0x000F;
			break;
		case dsFaultReset	:
			InOut.Controlword = ( InOut.Controlword & 0xFF7F ) | 0x0080;
			break;
	}
	
	// Quel que soit le mode de marche :
	//---------------------------------------------------------------------------------------------
	if (allHalt ) InOut.Controlword = ( InOut.Controlword & 0xFEFF ) | 0x0100;
	else InOut.Controlword = ( InOut.Controlword & 0xFEFF  ); 
	
	// Mode Profile Position :
	//---------------------------------------------------------------------------------------------
	if ( In.ModesOfOperationDisplay == 1 )
	{
		// bit 4 : new set point
		if ( ppNewSetPoint ) InOut.Controlword = ( InOut.Controlword & 0xFFEF ) | 0x0010;
		else InOut.Controlword = ( InOut.Controlword & 0xFFEF );
		
		// bit 5 : change set immediately
		if ( ppChangeSetImmediately )  InOut.Controlword = ( InOut.Controlword & 0xFFDF ) | 0x0020;
		else InOut.Controlword = ( InOut.Controlword & 0xFFDF );
		
		// bit 6 : 
		if ( ppChangeSetImmediately )  InOut.Controlword = ( InOut.Controlword & 0xFFBF ) | 0x0040;
		else InOut.Controlword = ( InOut.Controlword & 0xFFBF );		
		
	}
	else	// on reset les commandes
	{
		ppNewSetPoint = false;
		ppRelative = false;
		ppChangeSetImmediately = false;
	}
	
	// Mode Profile Velocity :
	//---------------------------------------------------------------------------------------------
	if ( In.ModesOfOperationDisplay == 3 )
	{
		
	}
	else
	{
		
	}
			
	// Ecriture des sorties TOR :
	//---------------------------------------------------------------------------------------------
	if ( OP1 ) SetDigitalOutput( 1 );
	else ClearDigitalOutput( 1 );
	//---------------------------------------------------------------------------------------------
	if ( OP2 ) SetDigitalOutput( 2 );
	else ClearDigitalOutput( 2 );
	
	// Ecriture dans les param�tres de Customer Storage (2701h)
	//---------------------------------------------------------------------------------------------
	InOut.Storage1 = move_step_act;	 
	InOut.Storage2 = dsSTATE;
}